[Asterisk-cvs] asterisk/apps app_mp3.c,1.16,1.17 app_nbscat.c,1.5,1.6

markster at lists.digium.com markster at lists.digium.com
Sat May 1 09:56:14 CDT 2004


Update of /usr/cvsroot/asterisk/apps
In directory mongoose.digium.com:/tmp/cvs-serv29236/apps

Modified Files:
	app_mp3.c app_nbscat.c 
Log Message:
Improve mp3 player quality (bug #1527)


Index: app_mp3.c
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/app_mp3.c,v
retrieving revision 1.16
retrieving revision 1.17
diff -u -d -r1.16 -r1.17
--- app_mp3.c	26 Apr 2004 03:02:49 -0000	1.16
+++ app_mp3.c	1 May 2004 14:03:37 -0000	1.17
@@ -104,15 +104,14 @@
 	int ms = -1;
 	int pid = -1;
 	int owriteformat;
-	struct timeval last;
+	struct timeval now, next;
 	struct ast_frame *f;
 	struct myframe {
 		struct ast_frame f;
 		char offset[AST_FRIENDLY_OFFSET];
 		short frdata[160];
 	} myf;
-	last.tv_usec = 0;
-	last.tv_sec = 0;
+
 	if (!data) {
 		ast_log(LOG_WARNING, "MP3 Playback requires an argument (filename)\n");
 		return -1;
@@ -131,35 +130,23 @@
 		return -1;
 	}
 	
+	gettimeofday(&now, NULL);
 	res = mp3play((char *)data, fds[1]);
 	/* Wait 1000 ms first */
-	ms = 1000;
+	next = now;
+	next.tv_sec += 1;
 	if (res >= 0) {
 		pid = res;
 		/* Order is important -- there's almost always going to be mp3...  we want to prioritize the
 		   user */
 		for (;;) {
-			ms = ast_waitfor(chan, ms);
-			if (ms < 0) {
-				ast_log(LOG_DEBUG, "Hangup detected\n");
-				res = -1;
-				break;
-			}
-			if (ms > 40) {
-				f = ast_read(chan);
-				if (!f) {
-					ast_log(LOG_DEBUG, "Null frame == hangup() detected\n");
-					res = -1;
-					break;
-				}
-				if (f->frametype == AST_FRAME_DTMF) {
-					ast_log(LOG_DEBUG, "User pressed a key\n");
-					ast_frfree(f);
-					res = 0;
-					break;
-				}
-				ast_frfree(f);
-			} else  {
+			gettimeofday(&now, NULL);
+			ms = (next.tv_sec - now.tv_sec) * 1000;
+			ms += (next.tv_usec - now.tv_usec) / 1000;
+#if 0
+			printf("ms: %d\n", ms);
+#endif			
+			if (ms <= 0) {
 #if 0
 				{
 					static struct timeval last;
@@ -190,10 +177,36 @@
 					res = 0;
 					break;
 				}
-				ms += res / 16;
+				next.tv_usec += res / 2 * 125;
+				if (next.tv_usec >= 1000000) {
+					next.tv_usec -= 1000000;
+					next.tv_sec++;
+				}
 #if 0
 				printf("Next: %d\n", ms);
 #endif				
+			} else {
+				ms = ast_waitfor(chan, ms);
+				if (ms < 0) {
+					ast_log(LOG_DEBUG, "Hangup detected\n");
+					res = -1;
+					break;
+				}
+				if (ms) {
+					f = ast_read(chan);
+					if (!f) {
+						ast_log(LOG_DEBUG, "Null frame == hangup() detected\n");
+						res = -1;
+						break;
+					}
+					if (f->frametype == AST_FRAME_DTMF) {
+						ast_log(LOG_DEBUG, "User pressed a key\n");
+						ast_frfree(f);
+						res = 0;
+						break;
+					}
+					ast_frfree(f);
+				} 
 			}
 		}
 	}

Index: app_nbscat.c
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/app_nbscat.c,v
retrieving revision 1.5
retrieving revision 1.6
diff -u -d -r1.5 -r1.6
--- app_nbscat.c	26 Apr 2004 03:02:49 -0000	1.5
+++ app_nbscat.c	1 May 2004 14:03:37 -0000	1.6
@@ -91,15 +91,13 @@
 	int ms = -1;
 	int pid = -1;
 	int owriteformat;
-	struct timeval last;
+	struct timeval now, next;
 	struct ast_frame *f;
 	struct myframe {
 		struct ast_frame f;
 		char offset[AST_FRIENDLY_OFFSET];
 		short frdata[160];
 	} myf;
-	last.tv_usec = 0;
-	last.tv_sec = 0;
 	if (socketpair(AF_LOCAL, SOCK_STREAM, 0, fds)) {
 		ast_log(LOG_WARNING, "Unable to create socketpair\n");
 		return -1;
@@ -116,55 +114,80 @@
 	
 	res = NBScatplay(fds[1]);
 	/* Wait 1000 ms first */
-	ms = 1000;
+	next = now;
+	next.tv_sec += 1;
 	if (res >= 0) {
 		pid = res;
 		/* Order is important -- there's almost always going to be NBScat...  we want to prioritize the
 		   user */
 		for (;;) {
-			ms = ast_waitfor(chan, ms);
-			if (ms < 0) {
-				ast_log(LOG_DEBUG, "Hangup detected\n");
-				res = -1;
-				break;
-			}
-			if (ms > 40) {
-				f = ast_read(chan);
-				if (!f) {
-					ast_log(LOG_DEBUG, "Null frame == hangup() detected\n");
-					res = -1;
-					break;
-				}
-				if (f->frametype == AST_FRAME_DTMF) {
-					ast_log(LOG_DEBUG, "User pressed a key\n");
-					ast_frfree(f);
-					res = 0;
-					break;
+			gettimeofday(&now, NULL);
+			ms = (next.tv_sec - now.tv_sec) * 1000;
+			ms += (next.tv_usec - now.tv_usec) / 1000;
+#if 0
+			printf("ms: %d\n", ms);
+#endif			
+			if (ms <= 0) {
+#if 0
+				{
+					static struct timeval last;
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					printf("Since last: %ld\n", (tv.tv_sec - last.tv_sec) * 1000 + (tv.tv_usec - last.tv_usec) / 1000);
+					last = tv;
 				}
-				ast_frfree(f);
-			} else  {
+#endif
 				res = timed_read(fds[0], myf.frdata, sizeof(myf.frdata));
 				if (res > 0) {
 					myf.f.frametype = AST_FRAME_VOICE;
 					myf.f.subclass = AST_FORMAT_SLINEAR;
 					myf.f.datalen = res;
 					myf.f.samples = res / 2;
-					myf.f.delivery.tv_usec = 0;
-					myf.f.delivery.tv_sec = 0;
 					myf.f.mallocd = 0;
 					myf.f.offset = AST_FRIENDLY_OFFSET;
 					myf.f.src = __PRETTY_FUNCTION__;
+					myf.f.delivery.tv_sec = 0;
+					myf.f.delivery.tv_usec = 0;
 					myf.f.data = myf.frdata;
 					if (ast_write(chan, &myf.f) < 0) {
 						res = -1;
 						break;
 					}
 				} else {
-					ast_log(LOG_DEBUG, "No more NBScat\n");
+					ast_log(LOG_DEBUG, "No more mp3\n");
 					res = 0;
 					break;
 				}
-				ms += res / 16;
+				next.tv_usec += res / 2 * 125;
+				if (next.tv_usec >= 1000000) {
+					next.tv_usec -= 1000000;
+					next.tv_sec++;
+				}
+#if 0
+				printf("Next: %d\n", ms);
+#endif				
+			} else {
+				ms = ast_waitfor(chan, ms);
+				if (ms < 0) {
+					ast_log(LOG_DEBUG, "Hangup detected\n");
+					res = -1;
+					break;
+				}
+				if (ms) {
+					f = ast_read(chan);
+					if (!f) {
+						ast_log(LOG_DEBUG, "Null frame == hangup() detected\n");
+						res = -1;
+						break;
+					}
+					if (f->frametype == AST_FRAME_DTMF) {
+						ast_log(LOG_DEBUG, "User pressed a key\n");
+						ast_frfree(f);
+						res = 0;
+						break;
+					}
+					ast_frfree(f);
+				} 
 			}
 		}
 	}
@@ -116,55 +114,80 @@
 	if (res >= 0) {
 		pid = res;
-		/* Order is important -- there's almost always going to be NBScat...  we want to prioritize the
+		/* Order is important -- there's almost always going to be mp3...  we want to prioritize the
 		   user */
 		for (;;) {




More information about the svn-commits mailing list